@@ -4,7 +4,7 @@ use Exporter 'import';
 use Digest::MD5 qw(md5_hex);
 
 use 5.010;
-our $VERSION = '1.616'; # ==> ALSO update the version in the pod text below!
+our $VERSION = '1.617'; # ==> ALSO update the version in the pod text below!
 
 # loaded modules
 our %TAG_HANDLERS;
@@ -27,7 +27,7 @@ XSLoader::load('DTL::Fast', $VERSION);
 
 our $RUNTIME_CACHE;
 
-our @EXPORT_OK;
+our @EXPORT_OK = qw(count_lines);
 
 push @EXPORT_OK, 'get_template';
 sub get_template
@@ -333,7 +333,7 @@ DTL::Fast - Perl implementation of Django templating language.
 
 =head1 VERSION
 
-Version 1.616
+Version 1.617
 
 =head1 SYNOPSIS
 
@@ -564,13 +564,24 @@ Works exactly like a perl's sprintf function with pattern and substitutions. Thi
 
 C<url> tag works a different way. Because there is no framework around, we can't obtain model's path the same way. But you may pass C<url_source> parameter into template constructor or C<get_template>/C<select_template> function. This parameter MUST be a reference to a function, that will return to templating engine url template by some 'model path' (first parameter of C<url> tag). Second parameter passed to the C<url_source> handler will be a reference to array of argument values (in case of positional arguments) or reference to a hash of arguments (in case of named ones). Url source handler may just return a regexp template by model path and templating engine will try to restore it with specified arguments. Or, you may restore it yourself, alter replacement arguments or do whatever you want. 
 
-=head3 warn
+=head3 dump
 
-    {% warn var1 var2 ... varn %}
+    {% dump var1 var2 ... varn %}
 
-C<warn> tag is useful for development and debugging. Dumps variable using L<C<Data::Dumper>> to C<STDERR>.
+C<dump> tag is useful for development and debugging. Dumps context variables using L<C<Data::Dumper>> to the rendered template as is.
+
+=head3 dump_html
+
+    {% dump_html var1 var2 ... varn %}
+
+Works exactly as the C<dump> tag, but escapes result and writes it to the rendered template, wrapped with C<textarea> tags. Textarea has C<dtl_fast_dump_area> class selector, so you may do any styling on your side. Useful for debugging HTML pages.
+
+=head3 dump_warn
+
+    {% dump_warn var1 var2 ... varn %}
+
+Works exactly as the C<dump> tag, warns output, instead of putting it into the rendered template.
 
-Without and argument dumps full context object.
 
 =head2 Filters
 
@@ -722,9 +733,10 @@ Every block tag in the library implemented as class, inherited from L<C<DTL::Fas
     sub parse_tag_chunk
     {
         my(
-            $self,      # reference to the tag object
-            $tag_name,  # tag keyword
-            $tag_param  # everything after tag keyword (constructor's $parameter)
+            $self,       # reference to the tag object
+            $tag_name,   # tag keyword
+            $tag_param,  # everything after tag keyword (constructor's $parameter)
+            $chunk_lines # chunk size in lines, used for proper debugging output
         ) = @_;
         
         my $result;
@@ -732,15 +744,17 @@ Every block tag in the library implemented as class, inherited from L<C<DTL::Fas
         # here you may interpret additional tag keywords, like else, elsif and so on.
         if( $tag_name eq 'my_tag_else' )
         {
-            ... do smth special 
+            ... do smth special
+            $DTL::Fast::Template::CURRENT_TEMPLATE_LINE += $chunk_lines; # set proper line number for next source block
         }
         elsif( $tag_name eq 'my_tag_alternative_end' )
         {
             $self->{'raw_chunks'} = []; # this construction ends current block parsing
+            $DTL::Fast::Template::CURRENT_TEMPLATE_LINE += $chunk_lines; # set proper line number for next source block
         }
         else    # if it was not special keyword, just do regular work
         {
-            $result = $self->SUPER::parse_tag_chunk($tag_name, $tag_param);
+            $result = $self->SUPER::parse_tag_chunk($tag_name, $tag_param, $chunk_lines);
         }
         
         return $result;  # result must be a generated chunk or undef if there is no one
