@@ -2,7 +2,7 @@ package DTL::Fast::Expression::Operator::Binary::Le;
 use strict; use utf8; use warnings FATAL => 'all'; 
 use parent 'DTL::Fast::Expression::Operator::Binary::Ge';
 
-$DTL::Fast::Expression::Operator::KNOWN{'<='} = __PACKAGE__;
+$DTL::Fast::OPS_HANDLERS{'<='} = __PACKAGE__;
 
 sub dispatch
 {
