@@ -1,5 +1,6 @@
 package DTL::Fast::Variable;
-use strict; use utf8; use warnings FATAL => 'all'; 
+use strict; use utf8; use warnings FATAL => 'all';
+use parent 'DTL::Fast::Replacer';
 no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
 use Scalar::Util qw(looks_like_number);
@@ -10,12 +11,14 @@ use DTL::Fast::Template;
 sub new
 {
     my( $proto, $variable, %kwargs ) = @_;
+
+    $proto = ref $proto || $proto;
     
     $variable =~ s/(^\s+|\s+$)//gsi;
     my @filters = split /\s*\|+\s*/, $variable;
     
     my $variable_name = shift @filters;
-
+    
     if( 
         $kwargs{'replacement'} 
         and my $replacement = $kwargs{'replacement'}->get_replacement($variable_name)
@@ -59,21 +62,28 @@ sub new
     }
     else
     {
-        die "Variable can't contain brackets: $variable_name" 
-            if $variable_name =~ /[()]/;
+        if ( $variable_name =~ /[^\w\-\.]/)
+        {
+            die $proto->get_parse_error(
+                "variable `$variable_name` contains incorrect symbols (not /alphanumeric/-/_/./ )"
+                , <<'_EOM_'
+      Possible reasons: typo in variable name
+                        typo in logical operator `=` instead of `==`, for example
+_EOM_
+            );
+        }
         @variable = split /\.+/, $variable_name;
     }
     
-    my $self = bless {
+    my $self = $proto->SUPER::new(
         'variable' => [@variable]
         , 'original' => $variable
         , 'direct_read' => ( scalar @variable == 1 )
         , 'sign' => $sign
         , 'undef' => $undef
         , 'static' => $static
-        , 'source_line' => $DTL::Fast::Template::CURRENT_TEMPLATE_LINE
         , 'filter_manager' => DTL::Fast::FilterManager->new('replacement' => $kwargs{'replacement'})
-    }, $proto;
+    );
 
     if( scalar @filters )
     {
